<?php
/**
 * Zikula Application Framework
 *
 * @copyright  (c) Zikula Development Team
 * @link       http://www.zikula.org
 * @version    $Id: past.php 75 2009-02-24 04:51:52Z mateo $
 * @license    GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author     Mark West <mark@zikula.org>
 * @category   Zikula_3rdParty_Modules
 * @package    Content_Management
 * @subpackage News
 */

/**
 * initialise block
 *
 * @author       The Zikula Development Team
 */
function News_pastblock_init()
{
    // Security
    SecurityUtil::registerPermissionSchema('Pastblock::', 'Block title::');
}

/**
 * get information on block
 *
 * @author       The Zikula Development Team
 * @return       array       The block information
 */
function News_pastblock_info()
{
    return array('text_type'      => 'Past',
                 'module'         => 'News',
                 'text_type_long' => 'Past Articles',
                 'allow_multiple' => true,
                 'form_content'   => false,
                 'form_refresh'   => false,
                 'show_preview'   => true);
}

/**
 * display block
 *
 * @author       The Zikula Development Team
 * @param        array       $blockinfo     a blockinfo structure
 * @return       output      the rendered bock
 */
function News_pastblock_display($blockinfo)
{
    // security check
    if (!SecurityUtil::checkPermission('Pastblock::', "$blockinfo[title]::", ACCESS_READ)) {
        return;
    }

    // get the number of stories shown on the frontpage
    $storyhome = pnModGetVar('News', 'storyhome', 10);

    // Break out options from our content field
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // Defaults
    if (empty($vars['limit'])) {
        $vars['limit'] = 10;
    }

    // call the API
    $articles = pnModAPIFunc('News', 'user', 'getall',
                             array('ihome'    => 0,
                                   'order'    => 'from',
                                   'status'   => 0,
                                   'startnum' => $storyhome,
                                   'numitems' => $vars['limit']));

    if ($articles === false) {
        return;
    }

    // loop round the return articles grouping by date
    $count        = 0;
    $news         = array();
    $newscumul    = array();
    $limitreached = false;
    foreach ($articles as $article) {
        $info  = pnModAPIFunc('News', 'user', 'getArticleInfo', $article);
        $links = pnModAPIFunc('News', 'user', 'getArticleLinks', $info);
        if (SecurityUtil::checkPermission('Stories::Story', "$info[aid]::$info[sid]", ACCESS_READ)) {
            $preformat['title'] = "<a href=\"$links[fullarticle]\">$info[title]</a>";
        } else {
            $preformat['title'] = $info['title'];
        }

        $daydate = DateUtil::formatDatetime(strtotime($info['from']), '%Y-%m-%d');

        // Reset the time
        if (!isset($currentday)) {
            $currentday = $daydate;
        }

        // If it's a different date, save the cumul and continue
        if ($currentday != $daydate) {
            $news[$currentday] = $newscumul;
            $newscumul = array();
            $currentday = $daydate;
        }
        $newscumul[] = array('info'      => $info,
                             'links'     => $links,
                             'preformat' => $preformat);
    }
    if (!isset($news[$currentday])) {
        $news[$currentday] = $newscumul;
    }

    $renderer = pnRender::getInstance('News');
    $renderer->assign('news', $news);

    if (empty($blockinfo['title'])) {
        $blockinfo['title'] = _PASTARTICLES;
    }
    $blockinfo['content'] = $renderer->fetch('news_block_past.htm');

    return pnBlockThemeBlock($blockinfo);
}

/**
 * modify block settings
 *
 * @author       The Zikula Development Team
 * @param        array       $blockinfo     a blockinfo structure
 * @return       output      the bock form
 */
function News_pastblock_modify($blockinfo)
{
    // Break out options from our content field
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // Defaults
    if (empty($vars['limit'])) {
        $vars['limit'] = 10;
    }

    // Create output object
    $renderer = pnRender::getInstance('News');

    // As Admin output changes often, we do not want caching.
    $renderer->caching = false;

    // assign the approriate values
    $renderer->assign($vars);

    // Return the output that has been generated by this function
    return $renderer->fetch('news_block_past_modify.htm');
}

/**
 * update block settings
 *
 * @author       The Zikula Development Team
 * @param        array       $blockinfo     a blockinfo structure
 * @return       $blockinfo  the modified blockinfo structure
 */
function News_pastblock_update($blockinfo)
{
    // Get current content
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // alter the corresponding variable
    $vars['limit'] = (int)FormUtil::getPassedValue('limit', null, 'POST');

    // write back the new contents
    $blockinfo['content'] = pnBlockVarsToContent($vars);

    // clear the block cache
    $renderer = pnRender::getInstance('News');
    $renderer->clear_cache('news_block_past.htm');

    return $blockinfo;
}
