<?php
/**
 * Zikula Application Framework
 *
 * @copyright  (c) Zikula Development Team
 * @link       http://www.zikula.org
 * @version    $Id: pntables.php 75 2009-02-24 04:51:52Z mateo $
 * @license    GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author     Mark West <mark@zikula.org>
 * @category   Zikula_3rdParty_Modules
 * @package    Content_Management
 * @subpackage News
 */

/**
 * Populate tables array for News module
 * 
 * This function is called internally by the core whenever the module is
 * loaded. It delivers the table information to the core.
 * It can be loaded explicitly using the pnModDBInfoLoad() API function.
 * 
 * @author       Xiaoyu Huang
 * @return       array       The table information.
 */
function News_pntables()
{
    // Initialise table array
    $tables = array();

    // Full table definition
    $tables['stories'] = DBUtil::getLimitedTablename('stories');
    $tables['stories_column'] = array(
        'sid'              => 'pn_sid',
        'aid'              => 'pn_cr_uid',   // for back compat
        'title'            => 'pn_title',
        'urltitle'         => 'pn_urltitle',
        'time'             => 'pn_cr_date',  // for back compat
        'hometext'         => 'pn_hometext',
        'bodytext'         => 'pn_bodytext',
        'counter'          => 'pn_counter',
        'informant'        => 'pn_informant',
        'notes'            => 'pn_notes',
        'ihome'            => 'pn_ihome',
        'language'         => 'pn_language',
        'withcomm'         => 'pn_withcomm',
        'format_type'      => 'pn_format_type',
        'published_status' => 'pn_published_status',
        'from'             => 'pn_from',
        'to'               => 'pn_to'
    );
    $tables['stories_column_def'] = array(
        'sid'              => 'I NOTNULL AUTO PRIMARY',
        'title'            => 'C(255) DEFAULT NULL',
        'urltitle'         => 'C(255) DEFAULT NULL',
        'hometext'         => 'X NOTNULL',
        'bodytext'         => 'X NOTNULL',
        'counter'          => 'I DEFAULT NULL',
        'informant'        => "C(20) NOTNULL DEFAULT ''",
        'notes'            => "X NOTNULL",
        'ihome'            => "I1 NOTNULL DEFAULT '0'",
        'language'         => "C(30) NOTNULL DEFAULT ''",
        'withcomm'         => "I1 NOTNULL DEFAULT '0'",
        'format_type'      => "I1 NOTNULL DEFAULT '0'",
        'published_status' => "I1 DEFAULT '0'",
        'from'             => 'T DEFAULT NULL',
        'to'               => 'T DEFAULT NULL'
    );

    // Enable categorization services
    $tables['stories_db_extra_enable_categorization'] = pnModGetVar('News', 'enablecategorization');
    $tables['stories_primary_key_column'] = 'sid';

    // Enable attribution services
    $tables['stories_db_extra_enable_attribution'] = pnModGetVar('News', 'enableattribution');

    // add standard data fields
    ObjectUtil::addStandardFieldsToTableDefinition ($tables['stories_column'], 'pn_');
    ObjectUtil::addStandardFieldsToTableDataDefinition($tables['stories_column_def']);

    // old comments table for upgrade
    $tables['comments'] = DBUtil::getLimitedTablename('comments');
    $tables['autonews'] = DBUtil::getLimitedTablename('autonews');
    $tables['queue'] = DBUtil::getLimitedTablename('queue');
    $tables['stories_cat'] = DBUtil::getLimitedTablename('stories_cat');
    $tables['topics'] = DBUtil::getLimitedTablename('topics');

    return $tables;
}
